@@ -3,29 +3,52 @@
 
 from __future__ import division, print_function
 
-import os
-from os.path import (join as pjoin, dirname, basename, exists, abspath,
-                     relpath, realpath)
+import functools
 import logging
+import os
 import shutil
 import warnings
-from subprocess import Popen, PIPE
-from typing import (Callable, Dict, FrozenSet, Iterable, List, Mapping,
-                    Optional, Set, Text, Tuple, Union)
+from os.path import abspath, basename, dirname, exists
+from os.path import join as pjoin
+from os.path import realpath, relpath
+from subprocess import PIPE, Popen
+from typing import (
+    Callable,
+    Dict,
+    FrozenSet,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Set,
+    Text,
+    Tuple,
+    Union,
+)
 
-from .pycompat import string_types
-from .libsana import (tree_libs, stripped_lib_dict, get_rp_stripper,
-                      walk_directory, get_dependencies)
-from .tools import (set_install_name, zip2dir, dir2zip, validate_signature,
-                    find_package_dirs, set_install_id, get_archs)
+from .libsana import (
+    _allow_all,
+    get_rp_stripper,
+    stripped_lib_dict,
+    tree_libs,
+    tree_libs_from_directory,
+)
 from .tmpdirs import TemporaryDirectory
-from .wheeltools import rewrite_record, InWheel
-
+from .tools import (
+    dir2zip,
+    find_package_dirs,
+    get_archs,
+    set_install_id,
+    set_install_name,
+    validate_signature,
+    zip2dir,
+)
+from .wheeltools import InWheel, rewrite_record
 
 logger = logging.getLogger(__name__)
 
 # Prefix for install_name_id of copied libraries
-DLC_PREFIX = '/DLC/'
+DLC_PREFIX = "/DLC/"
 
 
 class DelocationError(Exception):
@@ -35,10 +58,10 @@ class DelocationError(Exception):
 def delocate_tree_libs(
     lib_dict,  # type: Mapping[Text, Mapping[Text, Text]]
     lib_path,  # type: Text
-    root_path  # type: Text
+    root_path,  # type: Text
 ):
     # type: (...) -> Dict[Text, Dict[Text, Text]]
-    """ Move needed libraries in `lib_dict` into `lib_path`
+    """Move needed libraries in `lib_dict` into `lib_path`
 
     `lib_dict` has keys naming libraries required by the files in the
     corresponding value.  Call the keys, "required libs".  Call the values
@@ -94,10 +117,10 @@ def delocate_tree_libs(
 
 def _analyze_tree_libs(
     lib_dict,  # type: Mapping[Text, Mapping[Text, Text]]
-    root_path  # type: Text
+    root_path,  # type: Text
 ):
     # type: (...) -> Tuple[Dict[Text, Dict[Text, Text]], Set[Text]]
-    """ Verify then return which library files to copy and delocate.
+    """Verify then return which library files to copy and delocate.
 
     Returns
     -------
@@ -112,18 +135,21 @@ def _analyze_tree_libs(
     copied_basenames = set()
     rp_root_path = realpath(root_path)
     for required, requirings in lib_dict.items():
-        if required.startswith('@'):
+        if required.startswith("@"):
             # @rpath, etc, at this point should never happen.
             raise DelocationError("%s was expected to be resolved." % required)
         r_ed_base = basename(required)
-        if relpath(required, rp_root_path).startswith('..'):
+        if relpath(required, rp_root_path).startswith(".."):
             # Not local, plan to copy
             if r_ed_base in copied_basenames:
-                raise DelocationError('Already planning to copy library with '
-                                      'same basename as: ' + r_ed_base)
+                raise DelocationError(
+                    "Already planning to copy library with same basename as: "
+                    + r_ed_base
+                )
             if not exists(required):
-                raise DelocationError('library "{0}" does not exist'.format(
-                    required))
+                raise DelocationError(
+                    'library "{0}" does not exist'.format(required)
+                )
             # Copy requirings to preserve it since it will be modified later.
             needs_copying[required] = dict(requirings)
             copied_basenames.add(r_ed_base)
@@ -136,10 +162,10 @@ def _copy_required_libs(
     lib_dict,  # type: Mapping[Text, Mapping[Text, Text]]
     lib_path,  # type: Text
     root_path,  # type: Text
-    libraries_to_copy  # type: Iterable[Text]
+    libraries_to_copy,  # type: Iterable[Text]
 ):
     # type (...) -> Tuple[Dict[Text, Dict[Text, Text]], Set[Text]]
-    """ Copy libraries outside of root_path to lib_path.
+    """Copy libraries outside of root_path to lib_path.
 
     Returns
     -------
@@ -175,31 +201,39 @@ def _copy_required_libs(
 def _update_install_names(
     lib_dict,  # type: Mapping[Text, Mapping[Text, Text]]
     root_path,  # type: Text
-    files_to_delocate  # type: Iterable[Text]
+    files_to_delocate,  # type: Iterable[Text]
 ):
     # type: (...) -> None
-    """ Update the install names of libraries. """
+    """Update the install names of libraries."""
     for required in files_to_delocate:
         # Set relative path for local library
         for requiring, orig_install_name in lib_dict[required].items():
             req_rel = relpath(required, dirname(requiring))
-            new_install_name = '@loader_path/' + req_rel
-            logger.info(
-                "Modifying install name in %s from %s to %s",
-                relpath(requiring, root_path),
-                orig_install_name,
-                new_install_name,
-            )
-            set_install_name(requiring, orig_install_name, new_install_name)
+            new_install_name = "@loader_path/" + req_rel
+            if orig_install_name == new_install_name:
+                logger.info(
+                    "NOT modifying install name in %s from %s, as the new name"
+                    " would be the same.",
+                    relpath(requiring, root_path),
+                    orig_install_name,
+                )
+            else:
+                logger.info(
+                    "Modifying install name in %s from %s to %s",
+                    relpath(requiring, root_path),
+                    orig_install_name,
+                    new_install_name,
+                )
+                set_install_name(requiring, orig_install_name, new_install_name)
 
 
 def copy_recurse(
     lib_path,  # type: Text
     copy_filt_func=None,  # type: Optional[Callable[[Text], bool]]
-    copied_libs=None  # type: Optional[Dict[Text, Dict[Text, Text]]]
+    copied_libs=None,  # type: Optional[Dict[Text, Dict[Text, Text]]]
 ):
     # type: (...) -> Dict[Text, Dict[Text, Text]]
-    """ Analyze `lib_path` for library dependencies and copy libraries
+    """Analyze `lib_path` for library dependencies and copy libraries
 
     `lib_path` is a directory containing libraries.  The libraries might
     themselves have dependencies.  This function analyzes the dependencies and
@@ -256,10 +290,10 @@ def copy_recurse(
 def _copy_required(
     lib_path,  # type: Text
     copy_filt_func,  # type: Optional[Callable[[Text], bool]]
-    copied_libs  # type: Dict[Text, Dict[Text, Text]]
+    copied_libs,  # type: Dict[Text, Dict[Text, Text]]
 ):
     # type: (...) -> None
-    """ Copy libraries required for files in `lib_path` to `copied_libs`
+    """Copy libraries required for files in `lib_path` to `copied_libs`
 
     Augment `copied_libs` dictionary with any newly copied libraries, modifying
     `copied_libs` in-place - see Notes.
@@ -319,7 +353,7 @@ def _copy_required(
     for required, requirings in lib_dict.items():
         if copy_filt_func is not None and not copy_filt_func(required):
             continue
-        if required.startswith('@'):
+        if required.startswith("@"):
             # May have been processed by us, or have some rpath, loader_path of
             # its own. Either way, leave alone
             continue
@@ -328,9 +362,11 @@ def _copy_required(
         procd_requirings = {}
         # Set requiring lib install names to point to local copy
         for requiring, orig_install_name in requirings.items():
-            set_install_name(requiring,
-                             orig_install_name,
-                             '@loader_path/' + basename(required))
+            set_install_name(
+                requiring,
+                orig_install_name,
+                "@loader_path/" + basename(required),
+            )
             # Make processed version of ``dependings_dict``
             mapped_requiring = copied2orig.get(requiring, requiring)
             procd_requirings[mapped_requiring] = orig_install_name
@@ -341,22 +377,29 @@ def _copy_required(
         # Haven't see this one before, add entry to copied_libs
         out_path = pjoin(lib_path, basename(required))
         if exists(out_path):
-            raise DelocationError(out_path + ' already exists')
+            raise DelocationError(out_path + " already exists")
         shutil.copy(required, lib_path)
         copied2orig[out_path] = required
         copied_libs[required] = procd_requirings
 
 
-def _dylibs_only(filename):
-    # type: (Text) -> bool
-    return (filename.endswith('.so') or
-            filename.endswith('.dylib'))
+def _dylibs_only(filename: str) -> bool:
+    return filename.endswith(".so") or filename.endswith(".dylib")
 
 
-def filter_system_libs(libname):
-    # type: (Text) -> bool
-    return not (libname.startswith('/usr/lib') or
-                libname.startswith('/System'))
+def filter_system_libs(libname: str) -> bool:
+    return not (libname.startswith("/usr/lib") or libname.startswith("/System"))
+
+
+def _delocate_filter_function(
+    path: str,
+    *,
+    lib_filt_func: Callable[[str], bool],
+    copy_filt_func: Callable[[str], bool],
+) -> bool:
+    """Combines the library inspection and copy filters so that libraries
+    which won't be copied will not be followed."""
+    return lib_filt_func(path) and copy_filt_func(path)
 
 
 def delocate_path(
@@ -368,7 +411,7 @@ def delocate_path(
     ignore_missing=False,  # type: bool
 ):
     # type: (...) -> Dict[Text, Dict[Text, Text]]
-    """ Copy required libraries for files in `tree_path` into `lib_path`
+    """Copy required libraries for files in `tree_path` into `lib_path`
 
     Parameters
     ----------
@@ -387,6 +430,7 @@ def delocate_path(
         Default is callable rejecting only libraries beginning with
         ``/usr/lib`` or ``/System``.  None means copy all libraries. This will
         usually end up copying large parts of the system run-time.
+        Libraries which won't be copied will not be inspected for dependencies.
     executable_path : None or str, optional
         If not None, an alternative path to use for resolving
         `@executable_path`.
@@ -414,70 +458,118 @@ def delocate_path(
     elif isinstance(lib_filt_func, str):
         raise TypeError('lib_filt_func string can only be "dylibs-only"')
     if lib_filt_func is None:
-        lib_filt_func = (lambda _: True)
+        lib_filt_func = _allow_all
+    if copy_filt_func is None:
+        copy_filt_func = _allow_all
     if not exists(lib_path):
         os.makedirs(lib_path)
+    # Do not inspect dependencies of libraries that will not be copied.
+    filt_func = functools.partial(
+        _delocate_filter_function,
+        lib_filt_func=lib_filt_func,
+        copy_filt_func=copy_filt_func,
+    )
 
-    lib_dict = {}  # type: Dict[Text, Dict[Text, Text]]
-    missing_libs = False
-    for library_path in walk_directory(
-        tree_path, lib_filt_func, executable_path=executable_path
-    ):
-        for depending_path, install_name in get_dependencies(
-            library_path,
-            executable_path=executable_path,
-            filt_func=lib_filt_func,
-        ):
-            if depending_path is None:
-                missing_libs = True
-                continue
-            if copy_filt_func and not copy_filt_func(depending_path):
-                continue
-            lib_dict.setdefault(depending_path, {})
-            lib_dict[depending_path][library_path] = install_name
-
-    if missing_libs and not ignore_missing:
-        # Details of missing libraries would have already reported by
-        # get_dependencies.
-        raise DelocationError("Could not find all dependencies.")
+    lib_dict = tree_libs_from_directory(
+        tree_path,
+        lib_filt_func=filt_func,
+        copy_filt_func=filt_func,
+        executable_path=executable_path,
+        ignore_missing=ignore_missing,
+    )
 
     return delocate_tree_libs(lib_dict, lib_path, tree_path)
 
 
 def _copy_lib_dict(lib_dict):
     # type: (Mapping[Text, Mapping[Text, Text]]) -> Dict[Text, Dict[Text, Text]]  # noqa: E501
-    """ Returns a copy of lib_dict. """
+    """Returns a copy of lib_dict."""
     return {  # Convert nested Mapping types into nested Dict types.
         required: dict(requiring) for required, requiring in lib_dict.items()
     }
 
 
-def _merge_lib_dict(d1, d2):
-    # type: (Dict[Text, Dict[Text, Text]], Mapping[Text, Mapping[Text, Text]]) -> None  # noqa: E501
-    """ Merges lib_dict `d2` into lib_dict `d1`
+def _decide_dylib_bundle_directory(
+    wheel_dir: str, package_name: str, lib_sdir: str = ".dylibs"
+) -> str:
+    """Return a relative directory which should be used to store dylib files.
+
+    Parameters
+    ----------
+    wheel_dir : str
+        The directory of an unpacked wheel to analyse.
+    package_name : str
+        The name of the package.
+    lib_sdir : str, optional
+        Default value for lib sub-directory passed in via
+        :func:`delocate_wheel`.
+        Ignored if wheel has no package directories.
+
+    Returns
+    -------
+    dylibs_dir : str
+        A path to within `wheel_dir` where any library files should be put.
     """
-    for required, requirings in d2.items():
-        if required in d1:
-            d1[required].update(requirings)
-        else:
-            # Convert to dict to satisfy type checking.
-            d1[required] = dict(requirings)
-    return None
+    package_dirs = find_package_dirs(wheel_dir)
+    for directory in package_dirs:
+        if directory.endswith(package_name):
+            # Prefer using the directory with the same name as the package.
+            return pjoin(directory, lib_sdir)
+    if package_dirs:
+        # Otherwise, store dylib files in the first package alphabetically.
+        return pjoin(min(package_dirs), lib_sdir)
+    # Otherwise, use an auditwheel-style top-level name.
+    return pjoin(wheel_dir, f"{package_name}.dylibs")
+
+
+def _make_install_name_ids_unique(
+    libraries: Iterable[str], install_id_prefix: str
+) -> None:
+    """Replace each library's install name id with a unique id.
+
+    This is to change install ids to be unique within Python space.
+
+    Parameters
+    ----------
+    libraries : iterable of str
+        The libraries to be modified.
+        These files are assumed to be in the same directory.
+    install_id_prefix : str
+        A unique path to use as a prefix for the install name ids.
+        This must be a Unix absolute path.
+
+    Examples
+    --------
+    >>> _make_install_name_ids_unique((), "/")
+    >>> _make_install_name_ids_unique((), "")
+    Traceback (most recent call last):
+        ...
+    ValueError: install_id_prefix should start with '/', got ''
+    """
+    if not install_id_prefix.startswith("/"):
+        raise ValueError(
+            "install_id_prefix should start with '/',"
+            f" got {install_id_prefix!r}"
+        )
+    if not install_id_prefix.endswith("/"):
+        install_id_prefix += "/"
+    for lib in libraries:
+        set_install_id(lib, install_id_prefix + basename(lib))
+        validate_signature(lib)
 
 
 def delocate_wheel(
-    in_wheel,  # type: Text
-    out_wheel=None,  # type: Optional[Text]
-    lib_sdir='.dylibs',  # type: Text
-    lib_filt_func=None,  # type: Union[None, str, Callable[[Text], bool]]
-    copy_filt_func=filter_system_libs,  # type: Optional[Callable[[Text], bool]]
-    require_archs=None,  # type: Union[None, Text, Iterable[Text]]
-    check_verbose=False,  # type: bool
-    executable_path=None,  # type: Optional[Text]
-    ignore_missing=False,  # type: bool
-):
-    # type: (...) -> Dict[Text, Dict[Text, Text]]
-    """ Update wheel by copying required libraries to `lib_sdir` in wheel
+    in_wheel: str,
+    out_wheel: Optional[str] = None,
+    lib_sdir: str = ".dylibs",
+    lib_filt_func: Union[None, str, Callable[[str], bool]] = None,
+    copy_filt_func: Optional[Callable[[str], bool]] = filter_system_libs,
+    require_archs: Union[None, str, Iterable[str]] = None,
+    check_verbose: Optional[bool] = None,
+    executable_path: Optional[str] = None,
+    ignore_missing: bool = False,
+) -> Dict[str, Dict[str, str]]:
+    """Update wheel by copying required libraries to `lib_sdir` in wheel
 
     Create `lib_sdir` in wheel tree only if we are copying one or more
     libraries.
@@ -494,6 +586,8 @@ def delocate_wheel(
     lib_sdir : str, optional
         Subdirectory name in wheel package directory (or directories) to store
         needed libraries.
+       Ignored if the wheel has no package directories, and only contains
+       stand-alone modules.
     lib_filt_func : None or str or callable, optional
         If None, inspect all files for dependencies on dynamic libraries. If
         callable, accepts filename as argument, returns True if we should
@@ -514,10 +608,10 @@ def delocate_wheel(
         to sequence ``['x86_64, 'i386']``) or name of required architecture
         (e.g "i386" or "x86_64").
     check_verbose : bool, optional
-        If True, print warning messages about missing required architectures
-    executable_path : str, optional
+        This flag is deprecated, and has no effect.
+    executable_path : None or str, optional, keyword-only
         An alternative path to use for resolving `@executable_path`.
-    ignore_missing : bool, default=False
+    ignore_missing : bool, default=False, keyword-only
         Continue even if missing dependencies are detected.
 
     Returns
@@ -531,6 +625,13 @@ def delocate_wheel(
         is the ``install_name`` of ``copied_lib_path`` in the depending
         library. The filenames in the keys are relative to the wheel root path.
     """
+    if check_verbose is not None:
+        warnings.warn(
+            "The check_verbose flag is deprecated and shouldn't be provided,"
+            " all subsequent parameters should be changed over to keywords.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
     in_wheel = abspath(in_wheel)
     if out_wheel is None:
         out_wheel = in_wheel
@@ -538,55 +639,54 @@ def delocate_wheel(
         out_wheel = abspath(out_wheel)
     in_place = in_wheel == out_wheel
     with TemporaryDirectory() as tmpdir:
-        all_copied = {}  # type: Dict[Text, Dict[Text, Text]]
-        wheel_dir = realpath(pjoin(tmpdir, 'wheel'))
+        wheel_dir = realpath(pjoin(tmpdir, "wheel"))
         zip2dir(in_wheel, wheel_dir)
-        for package_path in find_package_dirs(wheel_dir):
-            lib_path = pjoin(package_path, lib_sdir)
-            lib_path_exists = exists(lib_path)
-            copied_libs = delocate_path(
-                package_path,
-                lib_path,
-                lib_filt_func,
-                copy_filt_func,
-                executable_path=executable_path,
-                ignore_missing=ignore_missing,
+        # Assume the package name from the wheel filename.
+        package_name = basename(in_wheel).split("-")[0]
+        lib_sdir = _decide_dylib_bundle_directory(
+            wheel_dir, package_name, lib_sdir
+        )
+        lib_path = pjoin(wheel_dir, lib_sdir)
+        lib_path_exists_before_delocate = exists(lib_path)
+        copied_libs = delocate_path(
+            wheel_dir,
+            lib_path,
+            lib_filt_func,
+            copy_filt_func,
+            executable_path=executable_path,
+            ignore_missing=ignore_missing,
+        )
+        if copied_libs and lib_path_exists_before_delocate:
+            raise DelocationError(
+                "f{lib_path} already exists in wheel but need to copy "
+                + "; ".join(copied_libs)
             )
-            if copied_libs and lib_path_exists:
+        if len(os.listdir(lib_path)) == 0:
+            shutil.rmtree(lib_path)
+        # Check architectures
+        if require_archs is not None:
+            bads = check_archs(copied_libs, require_archs)
+            if bads:
                 raise DelocationError(
-                    '{0} already exists in wheel but need to copy '
-                    '{1}'.format(lib_path, '; '.join(copied_libs)))
-            if len(os.listdir(lib_path)) == 0:
-                shutil.rmtree(lib_path)
-            # Check architectures
-            if require_archs is not None:
-                stop_fast = not check_verbose
-                bads = check_archs(copied_libs, require_archs, stop_fast)
-                if len(bads) != 0:
-                    if check_verbose:
-                        print(bads_report(bads, pjoin(tmpdir, 'wheel')))
-                    raise DelocationError(
-                        "Some missing architectures in wheel")
-            # Change install ids to be unique within Python space
-            install_id_root = (DLC_PREFIX +
-                               relpath(package_path, wheel_dir) +
-                               '/')
-            for lib in copied_libs:
-                lib_base = basename(lib)
-                copied_path = pjoin(lib_path, lib_base)
-                set_install_id(copied_path, install_id_root + lib_base)
-                validate_signature(copied_path)
-            _merge_lib_dict(all_copied, copied_libs)
-        if len(all_copied):
-            rewrite_record(wheel_dir)
-        if len(all_copied) or not in_place:
+                    "Some missing architectures in wheel"
+                    f"\n{bads_report(bads, pjoin(tmpdir, 'wheel'))}"
+                )
+        libraries_in_lib_path = [
+            pjoin(lib_path, basename(lib)) for lib in copied_libs
+        ]
+        _make_install_name_ids_unique(
+            libraries=libraries_in_lib_path,
+            install_id_prefix=DLC_PREFIX + relpath(lib_sdir, wheel_dir),
+        )
+        rewrite_record(wheel_dir)
+        if len(copied_libs) or not in_place:
             dir2zip(wheel_dir, out_wheel)
-    return stripped_lib_dict(all_copied, wheel_dir + os.path.sep)
+    return stripped_lib_dict(copied_libs, wheel_dir + os.path.sep)
 
 
 def patch_wheel(in_wheel, patch_fname, out_wheel=None):
     # type: (Text, Text, Optional[Text]) -> None
-    """ Apply ``-p1`` style patch in `patch_fname` to contents of `in_wheel`
+    """Apply ``-p1`` style patch in `patch_fname` to contents of `in_wheel`
 
     If `out_wheel` is None (the default), overwrite the wheel `in_wheel`
     in-place.
@@ -610,30 +710,27 @@ def patch_wheel(in_wheel, patch_fname, out_wheel=None):
     if not exists(patch_fname):
         raise ValueError("patch file {0} does not exist".format(patch_fname))
     with InWheel(in_wheel, out_wheel):
-        with open(patch_fname, 'rb') as fobj:
-            patch_proc = Popen(['patch', '-p1'],
-                               stdin=fobj,
-                               stdout=PIPE,
-                               stderr=PIPE)
+        with open(patch_fname, "rb") as fobj:
+            patch_proc = Popen(
+                ["patch", "-p1"], stdin=fobj, stdout=PIPE, stderr=PIPE
+            )
             stdout, stderr = patch_proc.communicate()
             if patch_proc.returncode != 0:
-                raise RuntimeError("Patch failed with stdout:\n" +
-                                   stdout.decode('latin1'))
+                raise RuntimeError(
+                    "Patch failed with stdout:\n" + stdout.decode("latin1")
+                )
 
 
-_ARCH_LOOKUP = {
-    'intel': ['i386', 'x86_64'],
-    'universal2': ['x86_64', 'arm64']
-}
+_ARCH_LOOKUP = {"intel": ["i386", "x86_64"], "universal2": ["x86_64", "arm64"]}
 
 
 def check_archs(
     copied_libs,  # type: Mapping[Text, Mapping[Text, Text]]
     require_archs=(),  # type:  Union[Text, Iterable[Text]]
-    stop_fast=False  # type: bool
+    stop_fast=False,  # type: bool
 ):
     # type: (...) -> Set[Union[Tuple[Text, FrozenSet[Text]], Tuple[Text, Text, FrozenSet[Text]]]]  # noqa: E501
-    """ Check compatibility of archs in `copied_libs` dict
+    """Check compatibility of archs in `copied_libs` dict
 
     Parameters
     ----------
@@ -672,10 +769,12 @@ def check_archs(
         ``depended_lib``.  An empty set means all architectures were present as
         required.
     """
-    if isinstance(require_archs, string_types):
+    if isinstance(require_archs, str):
         require_archs = _ARCH_LOOKUP.get(require_archs, [require_archs])
     require_archs_set = frozenset(require_archs)
-    bads = []  # type: List[Union[Tuple[Text, FrozenSet[Text]], Tuple[Text, Text, FrozenSet[Text]]]]  # noqa: E501
+    bads = (
+        []
+    )  # type: List[Union[Tuple[Text, FrozenSet[Text]], Tuple[Text, Text, FrozenSet[Text]]]]  # noqa: E501
     for depended_lib, dep_dict in copied_libs.items():
         depended_archs = get_archs(depended_lib)
         for depending_lib, install_name in dep_dict.items():
@@ -695,7 +794,7 @@ def check_archs(
 
 
 def bads_report(bads, path_prefix=None):
-    """ Return a nice report of bad architectures in `bads`
+    """Return a nice report of bad architectures in `bads`
 
     Parameters
     ----------
@@ -719,23 +818,30 @@ def bads_report(bads, path_prefix=None):
     report : str
         A nice report for printing
     """
-    path_processor = ((lambda x: x) if path_prefix is None
-                      else get_rp_stripper(path_prefix))
+    path_processor = (
+        (lambda x: x) if path_prefix is None else get_rp_stripper(path_prefix)
+    )
     reports = []
     for result in bads:
         if len(result) == 3:
             depended_lib, depending_lib, missing_archs = result
-            reports.append("{0} needs {1} {2} missing from {3}".format(
-                path_processor(depending_lib),
-                'archs' if len(missing_archs) > 1 else 'arch',
-                ', '.join(sorted(missing_archs)),
-                path_processor(depended_lib)))
+            reports.append(
+                "{0} needs {1} {2} missing from {3}".format(
+                    path_processor(depending_lib),
+                    "archs" if len(missing_archs) > 1 else "arch",
+                    ", ".join(sorted(missing_archs)),
+                    path_processor(depended_lib),
+                )
+            )
         elif len(result) == 2:
             depending_lib, missing_archs = result
-            reports.append("Required {0} {1} missing from {2}".format(
-                'archs' if len(missing_archs) > 1 else 'arch',
-                ', '.join(sorted(missing_archs)),
-                path_processor(depending_lib)))
+            reports.append(
+                "Required {0} {1} missing from {2}".format(
+                    "archs" if len(missing_archs) > 1 else "arch",
+                    ", ".join(sorted(missing_archs)),
+                    path_processor(depending_lib),
+                )
+            )
         else:
-            raise ValueError('Report tuple should be length 2 or 3')
-    return '\n'.join(sorted(reports))
+            raise ValueError("Report tuple should be length 2 or 3")
+    return "\n".join(sorted(reports))
