@@ -3,11 +3,13 @@
 Analyze library dependencies in paths and wheel files
 """
 
-import os
-from os.path import basename, dirname, join as pjoin, realpath
-import sys
-
 import logging
+import os
+import sys
+import warnings
+from os.path import basename, dirname
+from os.path import join as pjoin
+from os.path import realpath
 from typing import (
     Callable,
     Dict,
@@ -19,14 +21,16 @@ from typing import (
     Text,
     Tuple,
 )
-import warnings
 
-import six
+import delocate.delocating
 
-from .tools import (get_install_names, zip2dir, get_rpaths,
-                    get_environment_variable_paths)
 from .tmpdirs import TemporaryDirectory
-
+from .tools import (
+    get_environment_variable_paths,
+    get_install_names,
+    get_rpaths,
+    zip2dir,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -39,7 +43,7 @@ class DependencyNotFound(Exception):
 
 def _filter_system_libs(libname):
     # type: (Text) -> bool
-    return not (libname.startswith('/usr/lib') or libname.startswith('/System'))
+    return not (libname.startswith("/usr/lib") or libname.startswith("/System"))
 
 
 def get_dependencies(
@@ -93,7 +97,7 @@ def get_dependencies(
         if not _filter_system_libs(lib_fname):
             logger.debug(
                 "Ignoring missing library %s because it is a system library.",
-                lib_fname
+                lib_fname,
             )
             return
         raise DependencyNotFound(lib_fname)
@@ -114,7 +118,7 @@ def get_dependencies(
                     logger.debug(
                         "Skipped missing dependency %s"
                         " because it is a system library.",
-                        dependency_path
+                        dependency_path,
                     )
                 else:
                     raise DependencyNotFound(dependency_path)
@@ -124,8 +128,10 @@ def get_dependencies(
                 )
             yield dependency_path, install_name
         except DependencyNotFound:
-            message = "\n%s not found:\n  Needed by: %s" % (install_name,
-                                                            lib_fname)
+            message = "\n%s not found:\n  Needed by: %s" % (
+                install_name,
+                lib_fname,
+            )
             if install_name.startswith("@rpath"):
                 message += "\n  Search path:\n    " + "\n    ".join(rpaths)
             logger.error(message)
@@ -186,7 +192,9 @@ def walk_library(
     ):
         if dependency_fname is None:
             logger.error(
-                "%s not found, requested by %s", install_name, lib_fname,
+                "%s not found, requested by %s",
+                install_name,
+                lib_fname,
             )
             continue
         for sub_dependency in walk_library(
@@ -240,17 +248,160 @@ def walk_directory(
                 depending_path,
                 filt_func=filt_func,
                 visited=visited_paths,
-                executable_path=executable_path
+                executable_path=executable_path,
             ):
                 yield library_path
 
 
+def _tree_libs_from_libraries(
+    libraries: Iterable[str],
+    *,
+    lib_filt_func: Callable[[str], bool],
+    copy_filt_func: Callable[[str], bool],
+    executable_path: Optional[str] = None,
+    ignore_missing: bool = False,
+) -> Dict[str, Dict[str, str]]:
+    """Return an analysis of the dependencies of `libraries`.
+
+    Parameters
+    ----------
+    libraries : iterable of str
+        The paths to the libraries to find dependencies of.
+    lib_filt_func : callable, keyword-only
+        A callable which accepts filename as argument and returns True if we
+        should inspect the file or False otherwise.
+        If `filt_func` filters a library it will will not further analyze any
+        of that library's dependencies.
+    copy_filt_func : callable, keyword-only
+        Called on each library name detected as a dependency; copy
+        where ``copy_filt_func(libname)`` is True, don't copy otherwise.
+    executable_path : None or str, optional, keyword-only
+        If not None, an alternative path to use for resolving
+        `@executable_path`.
+    ignore_missing : bool, default=False, optional, keyword-only
+        Continue even if missing dependencies are detected.
+
+    Returns
+    -------
+    lib_dict : dict
+        dictionary with (key, value) pairs of (``libpath``,
+        ``dependings_dict``).
+
+        ``libpath`` is a canonical (``os.path.realpath``) filename of library,
+        or library name starting with {'@loader_path'}.
+
+
+        ``dependings_dict`` is a dict with (key, value) pairs of
+        (``depending_libpath``, ``install_name``), where ``dependings_libpath``
+        is the canonical (``os.path.realpath``) filename of the library
+        depending on ``libpath``, and ``install_name`` is the "install_name" by
+        which ``depending_libpath`` refers to ``libpath``.
+
+    Raises
+    ------
+    DelocationError
+        When any dependencies can not be located and ``ignore_missing`` is
+        False.
+    """
+    lib_dict: Dict[str, Dict[str, str]] = {}
+    missing_libs = False
+    for library_path in libraries:
+        for depending_path, install_name in get_dependencies(
+            library_path,
+            executable_path=executable_path,
+            filt_func=lib_filt_func,
+        ):
+            if depending_path is None:
+                missing_libs = True
+                continue
+            if copy_filt_func and not copy_filt_func(depending_path):
+                continue
+            lib_dict.setdefault(depending_path, {})
+            lib_dict[depending_path][library_path] = install_name
+
+    if missing_libs and not ignore_missing:
+        # get_dependencies will already have logged details of missing
+        # libraries.
+        raise delocate.delocating.DelocationError(
+            "Could not find all dependencies."
+        )
+
+    return lib_dict
+
+
+def tree_libs_from_directory(
+    start_path: str,
+    *,
+    lib_filt_func: Callable[[str], bool] = _filter_system_libs,
+    copy_filt_func: Callable[[str], bool] = lambda path: True,
+    executable_path: Optional[str] = None,
+    ignore_missing: bool = False,
+) -> Dict[Text, Dict[Text, Text]]:
+    """Return an analysis of the libraries in the directory of `start_path`.
+
+    Parameters
+    ----------
+    start_path : iterable of str
+        Root path of tree to search for libraries depending on other libraries.
+    lib_filt_func : callable, optional, keyword-only
+        A callable which accepts filename as argument and returns True if we
+        should inspect the file or False otherwise.
+        If `filt_func` filters a library it will will not further analyze any
+        of that library's dependencies.
+        Defaults to inspecting all files except for system libraries.
+    copy_filt_func : callable, optional, keyword-only
+        Called on each library name detected as a dependency; copy
+        where ``copy_filt_func(libname)`` is True, don't copy otherwise.
+        Defaults to copying all detected dependencies.
+    executable_path : None or str, optional, keyword-only
+        If not None, an alternative path to use for resolving
+        `@executable_path`.
+    ignore_missing : bool, default=False, optional, keyword-only
+        Continue even if missing dependencies are detected.
+
+    Returns
+    -------
+    lib_dict : dict
+        dictionary with (key, value) pairs of (``libpath``,
+        ``dependings_dict``).
+
+        ``libpath`` is a canonical (``os.path.realpath``) filename of library,
+        or library name starting with {'@loader_path'}.
+
+
+        ``dependings_dict`` is a dict with (key, value) pairs of
+        (``depending_libpath``, ``install_name``), where ``dependings_libpath``
+        is the canonical (``os.path.realpath``) filename of the library
+        depending on ``libpath``, and ``install_name`` is the "install_name" by
+        which ``depending_libpath`` refers to ``libpath``.
+
+    Raises
+    ------
+    DelocationError
+        When any dependencies can not be located and ``ignore_missing`` is
+        False.
+    """
+    return _tree_libs_from_libraries(
+        walk_directory(
+            start_path, lib_filt_func, executable_path=executable_path
+        ),
+        lib_filt_func=lib_filt_func,
+        copy_filt_func=copy_filt_func,
+        ignore_missing=ignore_missing,
+    )
+
+
+def _allow_all(path: str) -> bool:
+    """A filter which returns True for all files."""
+    return True
+
+
 def tree_libs(
     start_path,  # type: Text
     filt_func=None,  # type: Optional[Callable[[Text], bool]]
 ):
     # type: (...) -> Dict[Text, Dict[Text, Text]]
-    """ Return analysis of library dependencies within `start_path`
+    """Return analysis of library dependencies within `start_path`
 
     Parameters
     ----------
@@ -287,6 +438,8 @@ def tree_libs(
     .. deprecated:: 0.9
         This function does not support `@loader_path` and only returns the
         direct dependencies of the libraries in `start_path`.
+
+        :func:`tree_libs_from_directory` should be used instead.
     """
     warnings.warn(
         "tree_libs doesn't support @loader_path and has been deprecated.",
@@ -294,13 +447,14 @@ def tree_libs(
         stacklevel=2,
     )
     if filt_func is None:
-        filt_func = (lambda _: True)
+        filt_func = _allow_all
     lib_dict = {}  # type: Dict[Text, Dict[Text, Text]]
     for dirpath, dirnames, basenames in os.walk(start_path):
         for base in basenames:
             depending_path = realpath(pjoin(dirpath, base))
             for dependency_path, install_name in get_dependencies(
-                depending_path, filt_func=filt_func,
+                depending_path,
+                filt_func=filt_func,
             ):
                 if dependency_path is None:
                     # Mimic deprecated behavior.
@@ -311,7 +465,7 @@ def tree_libs(
                     # Support for `@loader_path` would break existing callers.
                     logger.debug(
                         "Excluding %s because it has '@loader_path'.",
-                        install_name
+                        install_name,
                     )
                     continue
                 lib_dict.setdefault(dependency_path, {})
@@ -321,7 +475,7 @@ def tree_libs(
 
 def resolve_dynamic_paths(lib_path, rpaths, loader_path, executable_path=None):
     # type: (Text, Iterable[Text], Text, Optional[Text]) -> Text
-    """ Return `lib_path` with any special runtime linking names resolved.
+    """Return `lib_path` with any special runtime linking names resolved.
 
     If `lib_path` has `@rpath` then returns the first `rpaths`/`lib_path`
     combination found.  If the library can't be found in `rpaths` then
@@ -357,14 +511,14 @@ def resolve_dynamic_paths(lib_path, rpaths, loader_path, executable_path=None):
     """
     if executable_path is None:
         executable_path = dirname(sys.executable)
-    if lib_path.startswith('@loader_path/'):
-        return realpath(pjoin(loader_path, lib_path.split('/', 1)[1]))
-    if lib_path.startswith('@executable_path/'):
-        return realpath(pjoin(executable_path, lib_path.split('/', 1)[1]))
-    if not lib_path.startswith('@rpath/'):
+    if lib_path.startswith("@loader_path/"):
+        return realpath(pjoin(loader_path, lib_path.split("/", 1)[1]))
+    if lib_path.startswith("@executable_path/"):
+        return realpath(pjoin(executable_path, lib_path.split("/", 1)[1]))
+    if not lib_path.startswith("@rpath/"):
         return realpath(lib_path)
 
-    lib_rpath = lib_path.split('/', 1)[1]
+    lib_rpath = lib_path.split("/", 1)[1]
     for rpath in rpaths:
         rpath_lib = resolve_dynamic_paths(
             pjoin(rpath, lib_rpath), (), loader_path, executable_path
@@ -377,7 +531,7 @@ def resolve_dynamic_paths(lib_path, rpaths, loader_path, executable_path=None):
 
 def resolve_rpath(lib_path, rpaths):
     # type: (Text, Iterable[Text]) -> Text
-    """ Return `lib_path` with its `@rpath` resolved
+    """Return `lib_path` with its `@rpath` resolved
     If the `lib_path` doesn't have `@rpath` then it's returned as is.
     If `lib_path` has `@rpath` then returns the first `rpaths`/`lib_path`
     combination found.  If the library can't be found in `rpaths` then a
@@ -403,10 +557,10 @@ def resolve_rpath(lib_path, rpaths):
         DeprecationWarning,
         stacklevel=2,
     )
-    if not lib_path.startswith('@rpath/'):
+    if not lib_path.startswith("@rpath/"):
         return lib_path
 
-    lib_rpath = lib_path.split('/', 1)[1]
+    lib_rpath = lib_path.split("/", 1)[1]
     for rpath in rpaths:
         rpath_lib = realpath(pjoin(rpath, lib_rpath))
         if os.path.exists(rpath_lib):
@@ -415,15 +569,15 @@ def resolve_rpath(lib_path, rpaths):
     warnings.warn(
         "Couldn't find {0} on paths:\n\t{1}".format(
             lib_path,
-            '\n\t'.join(realpath(path) for path in rpaths),
-            )
+            "\n\t".join(realpath(path) for path in rpaths),
         )
+    )
     return lib_path
 
 
 def search_environment_for_lib(lib_path):
     # type: (Text) -> Text
-    """ Search common environment variables for `lib_path`
+    """Search common environment variables for `lib_path`
 
     We'll use a single approach here:
 
@@ -450,32 +604,35 @@ def search_environment_for_lib(lib_path):
     Returns
     -------
     lib_path : str
-        Full path of ``basename(lib_path)``'s location, if it can be found, or
+        Real path of the first found location, if it can be found, or
         ``realpath(lib_path)`` if it cannot.
     """
     lib_basename = basename(lib_path)
     potential_library_locations = []
 
     # 1. Search on DYLD_LIBRARY_PATH
-    potential_library_locations += _paths_from_var('DYLD_LIBRARY_PATH',
-                                                   lib_basename)
+    potential_library_locations += _paths_from_var(
+        "DYLD_LIBRARY_PATH", lib_basename
+    )
 
     # 2. Search for realpath(lib_path)
     potential_library_locations.append(realpath(lib_path))
 
     # 3. Search on DYLD_FALLBACK_LIBRARY_PATH
-    potential_library_locations += \
-        _paths_from_var('DYLD_FALLBACK_LIBRARY_PATH', lib_basename)
+    potential_library_locations += _paths_from_var(
+        "DYLD_FALLBACK_LIBRARY_PATH", lib_basename
+    )
 
     for location in potential_library_locations:
         if os.path.exists(location):
-            return location
+            # See GH#133 for why we return the realpath here if it can be found
+            return realpath(location)
     return realpath(lib_path)
 
 
 def get_prefix_stripper(strip_prefix):
     # type: (Text) -> Callable[[Text], Text]
-    """ Return function to strip `strip_prefix` prefix from string if present
+    """Return function to strip `strip_prefix` prefix from string if present
 
     Parameters
     ----------
@@ -493,12 +650,13 @@ def get_prefix_stripper(strip_prefix):
     def stripper(path):
         # type: (Text) -> Text
         return path if not path.startswith(strip_prefix) else path[n:]
+
     return stripper
 
 
 def get_rp_stripper(strip_path):
     # type: (Text) -> Callable[[Text], Text]
-    """ Return function to strip ``realpath`` of `strip_path` from string
+    """Return function to strip ``realpath`` of `strip_path` from string
 
     Parameters
     ----------
@@ -517,7 +675,7 @@ def get_rp_stripper(strip_path):
 
 def stripped_lib_dict(lib_dict, strip_prefix):
     # type: (Dict[Text, Dict[Text, Text]], Text) -> Dict[Text, Dict[Text, Text]]
-    """ Return `lib_dict` with `strip_prefix` removed from start of paths
+    """Return `lib_dict` with `strip_prefix` removed from start of paths
 
     Use to give form of `lib_dict` that appears relative to some base path
     given by `strip_prefix`.  Particularly useful for analyzing wheels where we
@@ -550,11 +708,12 @@ def stripped_lib_dict(lib_dict, strip_prefix):
 
 
 def wheel_libs(
-    wheel_fname,  # type: Text
-    filt_func=None  # type: Optional[Callable[[Text], bool]]
-):
-    # type: (...) -> Dict[Text, Dict[Text, Text]]
-    """ Return analysis of library dependencies with a Python wheel
+    wheel_fname: str,
+    filt_func: Optional[Callable[[Text], bool]] = None,
+    *,
+    ignore_missing: bool = False,
+) -> Dict[Text, Dict[Text, Text]]:
+    """Return analysis of library dependencies with a Python wheel
 
     Use this routine for a dump of the dependency tree.
 
@@ -563,9 +722,11 @@ def wheel_libs(
     wheel_fname : str
         Filename of wheel
     filt_func : None or callable, optional
-        If None, inspect all files for library dependencies. If callable,
-        accepts filename as argument, returns True if we should inspect the
-        file, False otherwise.
+        If None, inspect all non-system files for library dependencies.
+        If callable, accepts filename as argument, returns True if we should
+        inspect the file, False otherwise.
+    ignore_missing : bool, default=False, optional, keyword-only
+        Continue even if missing dependencies are detected.
 
     Returns
     -------
@@ -576,16 +737,24 @@ def wheel_libs(
         is (key, value) of (``depending_lib_path``, ``install_name``).  Again,
         ``depending_lib_path`` is library relative to wheel root path, if
         within wheel tree.
+
+    Raises
+    ------
+    DelocationError
+        When dependencies can not be located and `ignore_missing` is False.
     """
+    if filt_func is None:
+        filt_func = _filter_system_libs
     with TemporaryDirectory() as tmpdir:
         zip2dir(wheel_fname, tmpdir)
-        lib_dict = tree_libs(tmpdir, filt_func)
+        lib_dict = tree_libs_from_directory(
+            tmpdir, lib_filt_func=filt_func, ignore_missing=ignore_missing
+        )
     return stripped_lib_dict(lib_dict, realpath(tmpdir) + os.path.sep)
 
 
-def _paths_from_var(varname, lib_basename):
-    # type: (Text, Text) -> List[Text]
-    var = os.environ.get(six.ensure_str(varname))
+def _paths_from_var(varname: str, lib_basename: str) -> List[str]:
+    var = os.environ.get(varname)
     if var is None:
         return []
-    return [pjoin(path, lib_basename) for path in var.split(':')]
+    return [pjoin(path, lib_basename) for path in var.split(":")]
